<?php

/**
 * @file
 * Module FAPI hooks and logic.
 */

/**
 * Helper function to get field columns with file ID.
 */
function _plyr_fid_item_columns() {
  $columns = &drupal_static(__FUNCTION__);

  if (!isset($columns)) {
    $columns = array(
      'fid',
      'poster_fid',
    );
  }
  return $columns;
}

/**
 * Implements hook_field_info().
 */
function plyr_field_info() {
  return array(
    'plyr' => array(
      'label' => t('Plyr media'),
      'description' => t('This field stores Multimedia file with attachements as fid integer values.'),
      'settings' => array(
        'uri_scheme' => variable_get('file_default_scheme', 'public'),
        'default_media' => 0,
      ),
      'instance_settings' => array(
        'file_extensions' => array(
          'media' => 'mp4 flv webm',
          'poster' => 'png gif jpg jpeg',
        ),
        'file_directory' => '',
        'max_filesize' => '',
      ),
      'default_widget' => 'plyr_media',
      'default_formatter' => 'plyr_player',
    ),
  );
}

/**
 * Implements hook_field_settings_form().
 */
function plyr_field_settings_form($field, $instance) {
  $defaults = field_info_field_settings($field['type']);
  $settings = array_merge($defaults, $field['settings']);

  $scheme_options = array();
  foreach (file_get_stream_wrappers(STREAM_WRAPPERS_WRITE_VISIBLE) as $scheme => $stream_wrapper) {
    $scheme_options[$scheme] = $stream_wrapper['name'];
  }
  $form['uri_scheme'] = array(
    '#type' => 'radios',
    '#title' => t('Upload destination'),
    '#options' => $scheme_options,
    '#default_value' => $settings['uri_scheme'],
    '#description' => t('Select where the final files should be stored. Private file storage has significantly more overhead than public files, but allows restricted access to files within this field.'),
  );

  return $form;
}

/**
 * Implements hook_field_instance_settings_form().
 */
function plyr_field_instance_settings_form($field, $instance) {
  $settings = $instance['settings'];

  // Use the file field instance settings form as a basis.
  $form = file_field_instance_settings_form($field, $instance);

  // Remove allowed extensions.
  unset($form['file_extensions']);

  // Remove the description option.
  unset($form['description_field']);

  return $form;
}

/**
 * Implements hook_field_load().
 */
function plyr_field_load($entity_type, $entities, $field, $instances, $langcode, &$items, $age) {
  foreach ($entities as $id => $entity) {
    foreach ($items[$id] as $delta => $item) {
      if (!empty($item['data'])) {
        $items[$id][$delta]['data'] = unserialize($item['data']);
      }
    }
  }
}

/**
 * Helper function to handle files of the field.
 */
function _plyr_field_handle_file(&$item, $path) {
  $fid = drupal_array_get_nested_value($item, $path);
  if ($fid) {
    $file = file_load($fid);
    if (empty($file)) {
      return FALSE;
    }
    elseif (!$file->status) {
      $file->status = FILE_STATUS_PERMANENT;
      file_save($file);
    }
  }
}

/**
 * Helper function to fetch poster images on field save.
 */
function _plyr_get_poster_images($field, $instance, &$items) {
  $destination_folder = file_field_widget_uri($field, $instance);
  file_prepare_directory($destination_folder, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS);
  foreach ($items as $delta => $item) {
    if ($item['source'] == 1 && empty($item['poster_fid'])) {
      $img_url = '';
      if (!empty($item['data']['url']) && $info = _plyr_get_stream_info($item['data']['url'])) {
        if ($info['type'] == 'youtube') {
          $img_url = 'http://img.youtube.com/vi/' . $info['src'] . '/hqdefault.jpg';
        }
        elseif ($info['type'] == 'vimeo') {
          $result = drupal_http_request('http://vimeo.com/api/v2/video/' . $info['src'] . '.php');
          if ($result->code == '200' && $data = unserialize($result->data)) {
            $img_url = $data[0]['thumbnail_large'];
          }
        }
      }

      if (!empty($img_url)) {
        $result = drupal_http_request($img_url);
        if ($result->code == '200' && !empty($result->data)) {
          $filepath = $destination_folder . '/' . basename($img_url);
          $file = file_save_data($result->data, $filepath);
          if ($file) {
            $item['poster_fid'] = $file->fid;
            $items[$delta] = $item;
          }
        }
      }
    }
  }
}

/**
 * Implements hook_field_presave().
 */
function plyr_field_presave($entity_type, $entity, $field, $instance, $langcode, &$items) {

  // Get poster images for external source videos.
  _plyr_get_poster_images($field, $instance, $items);

  // Make sure that each file which will be saved with this object has a
  // permanent status, so that it will not be removed when temporary files are
  // cleaned up.
  foreach ($items as $delta => $item) {
    foreach (_plyr_fid_item_columns() as $column) {
      _plyr_field_handle_file($item, array($column));
    }
    if (!empty($item['data'])) {
      $items[$delta]['data'] = serialize($item['data']);
    }
  }

}

/**
 * Helper function to add usage to files in sigle plyr field item.
 */
function _plyr_field_add_files_usage($item, $entity_type, $id) {
  foreach (_plyr_fid_item_columns() as $column) {
    if (!empty($item[$column])) {
      $file = file_load($item[$column]);
      file_usage_add($file, 'file', $entity_type, $id);
    }
  }
}

/**
 * Implements hook_field_insert().
 */
function plyr_field_insert($entity_type, $entity, $field, $instance, $langcode, &$items) {
  list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

  // Add a new usage of each uploaded file.
  foreach ($items as $item) {
    _plyr_field_add_files_usage($item, $entity_type, $id);
  }
}

/**
 * Implements hook_field_update().
 *
 * Checks for files that have been removed from the object.
 */
function plyr_field_update($entity_type, $entity, $field, $instance, $langcode, &$items) {
  // Check whether the field is defined on the object.
  if (!isset($entity->{$field['field_name']})) {
    // We cannot check for removed files if the field is not defined.
    return;
  }

  list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

  // On new revisions, all files are considered to be a new usage and no
  // deletion of previous file usages are necessary.
  if (!empty($entity->revision)) {
    foreach ($items as $item) {
      _plyr_field_add_files_usage($item);
    }
    return;
  }

  // Build a display of the current FIDs.
  $current_fids = array();
  foreach ($items as $item) {
    foreach (_plyr_fid_item_columns() as $column => $name) {
      if (!empty($item[$column])) {
        $current_fids[] = $item[$column];
      }
    }
  }

  // Compare the original field values with the ones that are being saved. Use
  // $entity->original to check this when possible, but if it isn't available,
  // create a bare-bones entity and load its previous values instead.
  if (isset($entity->original)) {
    $original = $entity->original;
  }
  else {
    $original = entity_create_stub_entity($entity_type, array(
      $id,
      $vid,
      $bundle,
    ));
    field_attach_load($entity_type, array($id => $original), FIELD_LOAD_CURRENT, array('field_id' => $field['id']));
  }
  $original_fids = array();
  if (!empty($original->{$field['field_name']}[$langcode])) {
    foreach ($original->{$field['field_name']}[$langcode] as $original_item) {
      foreach (_plyr_fid_item_columns() as $column => $name) {
        if (!empty($original_item[$column])) {
          $original_fids[] = $original_item[$column];
          if (isset($original_item[$column]) && !in_array($original_item[$column], $current_fids)) {
            // Decrement the file usage count by 1 and
            // delete the file if possible.
            file_field_delete_file($original_item[$name], $field, $entity_type, $id);
          }
        }
      }
    }
  }

  // Add new usage entries for newly added files.
  foreach ($items as $item) {
    foreach (_plyr_fid_item_columns() as $column) {
      if (!empty($item[$column]) && !in_array($item[$column], $original_fids)) {
        $file = file_load($item[$column]);
        file_usage_add($file, 'file', $entity_type, $id);
      }
    }
  }
}

/**
 * Implements hook_field_delete().
 */
function plyr_field_delete($entity_type, $entity, $field, $instance, $langcode, &$items) {
  list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

  // Delete all file usages within this entity.
  foreach ($items as $delta => $item) {
    if ($item['source'] !== 2) {
      continue;
    }
    foreach (_plyr_fid_item_columns() as $column) {
      $file = file_load($item[$column]);
      file_field_delete_file($file, $field, $entity_type, $id, 0);
    }
  }
}

/**
 * Implements hook_field_delete_revision().
 */
function plyr_field_delete_revision($entity_type, $entity, $field, $instance, $langcode, &$items) {
  list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);
  foreach ($items as $delta => $item) {
    if ($item['source'] !== 2) {
      continue;
    }

    // Decrement the file usage count by 1 and delete the file if possible.
    foreach (_plyr_fid_item_columns() as $column => $name) {
      $file = file_load($item[$column]);
      file_field_delete_file($file, $field, $entity_type, $id, 0);
    }
  }
}

/**
 * Implements hook_field_is_empty().
 */
function plyr_field_is_empty($item, $field) {
  if (empty($item['source'])) {
    return TRUE;
  }
  elseif ($item['source'] == 1) {
    if (empty($item['data']['url'])) {
      return TRUE;
    }
  }
  elseif ($item['source'] == 2) {
    if (empty($item['fid'])) {
      return TRUE;
    }
  }
  return FALSE;
}

/**
 * Implements hook_field_formatter_info().
 */
function plyr_field_formatter_info() {
  return array(
    'plyr_player' => array(
      'label' => t('Plyr player'),
      'field types' => array('plyr', 'file'),
      'settings' => array(
        'autoplay' => FALSE,
        'autoadvance' => TRUE,
        'loop' => FALSE,
      ),
    ),
    'plyr_poster' => array(
      'label' => t('Plyr poster'),
      'field types' => array('plyr'),
      'settings' => array('image_style' => '', 'image_link' => ''),
    ),
  );
}

/**
 * Implements hook_field_formatter_settings_form().
 */
function plyr_field_formatter_settings_form($field, $instance, $view_mode, $form, &$form_state) {
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];

  switch ($display['type']) {
    case 'plyr_poster':
      $image_styles = image_style_options(FALSE, PASS_THROUGH);
      $element['image_style'] = array(
        '#title' => t('Image style'),
        '#type' => 'select',
        '#default_value' => $settings['image_style'],
        '#empty_option' => t('None (original image)'),
        '#options' => $image_styles,
      );

      $link_types = array(
        'content' => t('Content'),
      );
      $element['image_link'] = array(
        '#title' => t('Link image to'),
        '#type' => 'select',
        '#default_value' => $settings['image_link'],
        '#empty_option' => t('Nothing'),
        '#options' => $link_types,
      );
      break;

    case 'plyr_player':
      $element['autoplay'] = array(
        '#type' => 'checkbox',
        '#title' => t('Autoplay'),
        '#default_value' => $settings['autoplay'],
      );
      if ($field['cardinality'] != 1) {
        $element['autoadvance'] = array(
          '#type' => 'checkbox',
          '#title' => t('Auto advance'),
          '#default_value' => $settings['autoadvance'],
        );
        $element['loop'] = array(
          '#type' => 'checkbox',
          '#title' => t('Loop'),
          '#default_value' => $settings['loop'],
        );
      }
      break;
  }
  return $element;
}

/**
 * Implements hook_field_formatter_settings_summary().
 */
function plyr_field_formatter_settings_summary($field, $instance, $view_mode) {
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];

  $summary = array();
  switch ($display['type']) {
    case 'plyr_poster':
      $image_styles = image_style_options(FALSE, PASS_THROUGH);
      // Unset possible 'No defined styles' option.
      unset($image_styles['']);
      // Styles could be lost because of enabled/disabled modules that defines
      // their styles in code.
      if (isset($image_styles[$settings['image_style']])) {
        $summary[] = t('Image style: @style', array('@style' => $image_styles[$settings['image_style']]));
      }
      else {
        $summary[] = t('Original image');
      }

      $link_types = array(
        'content' => t('Linked to content'),
      );
      // Display this setting only if image is linked.
      if (isset($link_types[$settings['image_link']])) {
        $summary[] = $link_types[$settings['image_link']];
      }
      break;

    case 'plyr_player':
      $summary[] = t('Autoplay: @autoplay', array(
        '@autoplay' => empty($settings['autoplay']) ? t('No') : t('Yes'),
      ));
      if ($field['cardinality'] != 1) {
        $summary[] = t('Autoadvance: @autoadvance', array(
          '@autoadvance' => empty($settings['autoadvance']) ? t('No') : t('Yes'),
        ));
        $summary[] = t('Loop: @loop', array(
          '@loop' => empty($settings['loop']) ? t('No') : t('Yes'),
        ));
      }
      break;

  }

  return implode('<br />', $summary);

}

/**
 * Heler function to get stream info.
 */
function _plyr_get_stream_info($url) {
  $data = array();
  if (strpos($url, 'youtu.be')) {
    $data['data_type'] = 'external';
    $data['source_type'] = 'video';
    $data['type'] = 'youtube';
    $data['src'] = trim(substr($url, strrpos($url, '/') + 1), '/');
  }
  elseif (strpos($url, 'youtube')) {
    $data['data_type']   = 'external';
    $data['source_type'] = 'video';
    $data['type']        = 'youtube';
    $data['src']         = trim(substr($url, strrpos($url, '=') + 1), '/');
  }
  elseif (strpos($url, 'vimeo.com')) {
    $data['data_type']   = 'external';
    $data['source_type'] = 'video';
    $data['type']        = 'vimeo';
    $data['src']         = trim(substr($url, strrpos($url, '/') + 1), '/');
  }
  else {
    $mime = file_get_mimetype($url);
    $type = substr($mime, 0, strpos($mime, '/'));
    if (in_array($type, array('video', 'audio'))) {
      $data['data_type'] = 'video';
      $data['source_type'] = $type;
      $data['type'] = $data['source_type'] . '/' . substr($url, strrpos($url, '.') + 1);
      $data['src'] = $url;
    }
  }
  if (!empty($data)) {
    return $data;
  }
  else {
    return FALSE;
  }
}

/**
 * Implements hook_field_formatter_view().
 */
function plyr_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {
  $element = array();
  $settings = $display['settings'];

  switch ($display['type']) {
    case 'plyr_player':
      $item = $items[0];

      $element = array(
        '#theme' => 'plyr',
        '#settings' => $settings,
      );

      // Prepare plyr sources.
      $element['#sources'] = array();
      foreach ($items as $delta => $item) {
        $element['#sources'][$delta] = array(
          'title' => isset($item['data']['title']) ? $item['data']['title'] : '',
          'poster' => isset($item['poster_fid']) ? file_load($item['poster_fid']) : NULL,
          'stream' => isset($item['data']['url']) ? $item['data']['url'] : NULL,
          'media_files' => isset($item['fid']) ? array(file_load($item['fid'])) : array(),
        );
      }
      break;

    case 'plyr_poster':
      if ($display['settings']['image_link'] == 'content') {
        $uri = entity_uri($entity_type, $entity);
      }

      foreach ($items as $delta => $item) {
        if (!empty($item['poster_fid'])) {
          $file = file_load($item['poster_fid']);
          $element[$delta] = array(
            '#theme' => 'image_formatter',
            '#item' => (array) $file,
            '#image_style' => $display['settings']['image_style'],
            '#path' => isset($uri) ? $uri : '',
          );
        }
      }
      break;

  }

  return $element;
}

/**
 * Implements hook_field_widget_info().
 */
function plyr_field_widget_info() {
  return array(
    'plyr_media' => array(
      'label' => t('Plyr multimedia'),
      'field types' => array('plyr'),
      'settings' => array(
        'progress_indicator' => 'throbber',
        'preview_poster_style' => 'thumbnail',
        'media_source' => 0,
      ),
      'behaviors' => array(
        'default value' => FIELD_BEHAVIOR_NONE,
      ),
    ),
  );
}

/**
 * Implements hook_field_widget_settings_form().
 */
function plyr_field_widget_settings_form($field, $instance) {
  $widget = $instance['widget'];
  $settings = $widget['settings'];

  // Use the file widget settings form.
  $form = file_field_widget_settings_form($field, $instance);

  $form['preview_poster_style'] = array(
    '#title' => t('Preview poster image style'),
    '#type' => 'select',
    '#options' => image_style_options(FALSE, PASS_THROUGH),
    '#empty_option' => '<' . t('no preview') . '>',
    '#default_value' => $settings['preview_poster_style'],
    '#description' => t('The preview image will be shown while editing the content.'),
    '#weight' => 15,
  );
  $form['media_source'] = array(
    '#title' => t('Possible media sources'),
    '#type' => 'select',
    '#options' => array(
      0 => t('All'),
      1 => t('External'),
      2 => t('Files'),
    ),
    '#default_value' => $settings['media_source'],
  );

  return $form;
}

/**
 * Implements hook_field_widget_form().
 */
function plyr_field_widget_form(&$form, &$form_state, $field, $instance, $langcode, $items, $delta, $element) {
  $widget_settings = $instance['widget']['settings'];

  // Get current values.
  if (!isset($form_state['storage'][$element['#field_name']][$langcode][$delta])) {

    if (!empty($items[$delta])) {
      $form_state['storage'][$element['#field_name']][$langcode][$delta] = $items[$delta];
    }
    else {
      $form_state['storage'][$element['#field_name']][$langcode][$delta] = array(
        'source' => $widget_settings['media_source'],
      );
    }
  }
  $values = &$form_state['storage'][$element['#field_name']][$langcode][$delta];

  if (isset($form_state['values'][$element['#field_name']][$langcode][$delta])) {
    $new_values = &$form_state['values'][$element['#field_name']][$langcode][$delta];
    foreach ($new_values as $key => $value) {
      $values[$key] = $value;
    }
  }

  $wrapper_id = 'plyr-instance-' . $instance['id'] . '-delta-' . $delta;
  $element += array(
    '#type' => 'fieldset',
    '#prefix' => '<div id = "' . $wrapper_id . '">',
    '#suffix' => '</div>',
    '#plyr_wrapper_id' => $wrapper_id,
  );

  $fields = array();

  if ($widget_settings['media_source']) {
    $fields['source'] = array(
      '#type' => 'value',
      '#value' => $widget_settings['media_source'],
    );
  }
  else {
    $fields['source'] = array(
      '#type' => 'select',
      '#title' => t('Media source'),
      '#options' => array(
        0 => t('None (field empty)'),
        1 => t('External'),
        2 => t('File'),
      ),
      '#ajax' => array(
        'callback' => 'plyr_field_widget_ajax',
      ),
      '#default_value' => isset($values['source']) ? $values['source'] : NULL,
    );
  }

  if ($field['cardinality'] != 1) {
    $fields['data']['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#description' => t('Title of the playlist item.'),
      '#default_value' => isset($values['data']['title']) ? $values['data']['title'] : NULL,
    );
  }

  $element_info = element_info('managed_file');
  $fields['poster_fid'] = array(
    '#title' => t('Poster image'),
    '#type' => 'managed_file',
    '#upload_location' => file_field_widget_uri($field, $instance),
    '#upload_validators' => array(
      'file_validate_extensions' => array('png gif jpg jpeg'),
    ),
    '#progress_indicator' => $instance['widget']['settings']['progress_indicator'],
    '#extended' => FALSE,
    '#process' => array_merge($element_info['#process'], array('plyr_field_widget_poster_process')),
    '#preview_image_style' => $widget_settings['preview_poster_style'],
    '#default_value' => isset($values['poster_fid']) ? $values['poster_fid'] : NULL,
    '#description' => t('If not uploaded and video is from Youtube or Vimeo, the service thumbnail will be used as poster image.'),
  );
  $fields['poster_fid']['#description'] = theme('file_upload_help', array(
    'description' => $fields['poster_fid']['#description'],
    'upload_validators' => $fields['poster_fid']['#upload_validators'],
  ));

  switch ($values['source']) {
    case 1:
      $fields['data']['url'] = array(
        '#type' => 'textfield',
        '#title' => t('External URL'),
        '#default_value' => empty($values['data']['url']) ? '' : $values['data']['url'],
        '#description' => t('Supported providers: Youtube, Vimeo. Can also be a path to an external media file.'),
        '#default_value' => isset($values['data']['url']) ? $values['data']['url'] : NULL,
      );
      break;

    case 2:
      $fields['fid'] = array(
        '#title' => t('Main media file'),
        '#type' => 'managed_file',
        '#upload_location' => file_field_widget_uri($field, $instance),
        '#upload_validators' => array(
          'file_validate_extensions' => array('mp4 flv webm mp3 ogg'),
        ),
        '#progress_indicator' => $instance['widget']['settings']['progress_indicator'],
        '#extended' => FALSE,
        '#default_value' => isset($values['fid']) ? $values['fid'] : NULL,
        '#description' => '',
      );
      $fields['fid']['#description'] = theme('file_upload_help', array(
        'description' => $fields['fid']['#description'],
        'upload_validators' => $fields['fid']['#upload_validators'],
      ));
      break;

  }

  $element += $fields;

  return $element;
}

/**
 * An element #process callback for the poster element.
 */
function plyr_field_widget_poster_process($element, &$form_state, $form) {
  // Add the image preview.
  if ($element['#file'] && $element['#preview_image_style']) {
    $variables = array(
      'style_name' => $element['#preview_image_style'],
      'path' => $element['#file']->uri,
    );

    $info = image_get_info($element['#file']->uri);

    if (is_array($info)) {
      $variables['width'] = $info['width'];
      $variables['height'] = $info['height'];
    }
    else {
      $variables['width'] = $variables['height'] = NULL;
    }

    $element['preview'] = array(
      '#type' => 'markup',
      '#markup' => theme('image_style', $variables),
    );
  }

  return $element;
}

/**
 * Type change AJAX callback.
 */
function plyr_field_widget_ajax($form, $form_state) {
  $commands = array();
  $array_parents = $form_state['triggering_element']['#array_parents'];
  array_pop($array_parents);
  $element = drupal_array_get_nested_value($form, $array_parents);
  hide($element['_weight']);
  $wrapper_id = $element['#plyr_wrapper_id'];

  $messages = theme('status_messages');
  if (!empty($messages)) {
    $element['#prefix'] .= $messages;
  }

  $commands[] = ajax_command_replace('#' . $wrapper_id, render($element));

  return array('#type' => 'ajax', '#commands' => $commands);
}
